/*****************************************************************************\
* (c) Copyright 2018-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the Apache License          *
* version 2 (Apache-2.0), copied verbatim in the file "LICENSE".              *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

#include <deque>
#include <map>
#include <string>
#include <memory>

#include "ROOTHeaders.h"

struct MonitorManager;

struct MonitorBase {
  enum MonHistType {
    MonitoringSuccess = 0,
    MonitoringSkipped,
    SplitSlices,
    MonitoringLevel0,
    MonitoringLevel1,
    MonitoringLevel2,
    MonitoringLevel3,
    MonitoringLevel4,
    MonitoringLevel5P,
    InclusiveRate = 100,
    LineRatesStart = 101,
    LineRatesLast = 199,
    KalmanTrackN = 200,
    KalmanTrackP,
    KalmanTrackPt,
    KalmanTrackEta,
    KalmanTrackIPChi2,
    PrimaryVertexN = 300,
    PrimaryVertexX,
    PrimaryVertexY,
    PrimaryVertexZ,
    SecondaryVertexN = 400,
    SecondaryVertexX,
    SecondaryVertexY,
    SecondaryVertexZ,
    SecondaryVertexPt,
    SecondaryVertexEta,
    SecondaryVertexMinPt,
    SecondaryVertexMinIPChi2,
    SecondaryVertexSumPt,
    SecondaryVertexMDiMu,
    SecondaryVertexMCor,
  };

  MonitorBase(MonitorManager* manager, std::string name, int timeStep, int offset) :
    m_manager {manager}, m_name(name), m_time_step(timeStep), m_offset(offset) {};

  virtual ~MonitorBase() = default;

  virtual void saveHistograms() const;

protected:
  unsigned getWallTimeBin();

  MonitorManager* m_manager = nullptr;
  std::string m_name;
  std::map<unsigned, std::unique_ptr<TH1>> m_histograms;

  unsigned m_time_step {};
  unsigned m_offset {};
};
