/*****************************************************************************\
* (c) Copyright 2018-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the Apache License          *
* version 2 (Apache-2.0), copied verbatim in the file "LICENSE".              *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "Logger.h"

#if defined(__clang__) && __clang_major__ >= 10
#pragma clang diagnostic push
#if !defined(__APPLE__)
#pragma clang diagnostic ignored "-Wdeprecated-copy"
#endif
#endif

#include "boost/iostreams/stream.hpp"

#if defined(__clang__) && __clang_major__ >= 10
#pragma clang diagnostic pop
#endif

#include "boost/iostreams/device/null.hpp"
#include <sstream>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>

namespace logger {
  static std::unique_ptr<Logger> ll;
  static std::unique_ptr<boost::iostreams::stream<boost::iostreams::null_sink>> nullOstream;
} // namespace logger

void logger::setVerbosity(int level)
{
  if (!logger::ll) {
    logger::ll.reset(new logger::Logger {});
  }
  logger::ll->verbosityLevel = level;
}

int logger::verbosity()
{
  if (!logger::ll) {
    logger::ll.reset(new logger::Logger {});
  }
  return logger::ll->verbosityLevel;
}

std::ostream& logger::logger(int requestedLogLevel)
{
  if (!logger::ll) {
    logger::ll.reset(new logger::Logger {});
  }
  if (!logger::nullOstream) {
    logger::nullOstream.reset(
      new boost::iostreams::stream<boost::iostreams::null_sink> {boost::iostreams::null_sink()});
  }
  if (logger::ll->verbosityLevel >= requestedLogLevel) {
    return std::cout;
  }
  else {
    return *logger::nullOstream;
  }
}
